//
//  main.cpp
//  DrawPath
//
//  Created by Nathan Block on 5/13/20.
//  Copyright © 2020 Nathan Block. All rights reserved.
//

#include <iostream>
#include <fstream>
#include <math.h>
#include <random>
#include <vector>
#include <iomanip>      // std::setprecision
#include <string>


#define PI 3.14159265358979

using namespace std;
void linear_time_step(double &x, double &y, double &angle, double & time, double speed, double dt, ofstream & true_flight_path){
    x += speed * dt * cos(angle);
    y += speed * dt * sin(angle);
    time += dt;
    true_flight_path << setprecision(10) << time << " " << x << " " << y << " " << speed << endl;
}
void generate_flight_path(){
    ofstream true_flight_path;
    true_flight_path.open ("true_flight_path.txt");
    double x, y, angle;
    double sampling_period = 0.25; //0.004kHz sampling frequency
    double time = 0;
    double linear_speed = 178; // 400 mph = 178 m/s
    double turning_radius = 10000; //10 km turning radius
    x = 100000; //start 100 km away from origin
    y = 0;
    angle = 150 * PI / 180;
    
    //fly straight at an angle of 150 degrees
    while(x > 60000){
        linear_time_step(x,y,angle,time, linear_speed,sampling_period,true_flight_path);
        linear_speed += 0.01 * sampling_period;
    }
    //arc 120 degrees upward
    while(angle > PI / 2){
        linear_time_step(x,y,angle,time, linear_speed,sampling_period,true_flight_path);
        angle -= linear_speed / turning_radius * sampling_period;
    }
    //fly vertical
    while(y < 80000){
        linear_time_step(x,y,angle,time, linear_speed,sampling_period,true_flight_path);
        linear_speed -= 0.02 * sampling_period;
    }
    //180 degree turn
    while(angle < 3 * PI / 2){
        linear_time_step(x,y,angle,time, linear_speed,sampling_period,true_flight_path);
        angle += linear_speed / turning_radius * sampling_period;
    }
    //fly vertical
    while(y > 50000){
        linear_time_step(x,y,angle,time, linear_speed,sampling_period,true_flight_path);
        linear_speed += 0.02 * sampling_period;
    }
    //turn to 330 degrees
    while(angle < 2 * PI * 11 / 12){
        linear_time_step(x,y,angle,time, linear_speed,sampling_period,true_flight_path);
        angle += linear_speed / turning_radius * sampling_period;
    }
    //fly straight
    while(x < 60000){
        linear_time_step(x,y,angle,time, linear_speed,sampling_period,true_flight_path);
        linear_speed += 0.01 * sampling_period;
    }
    //turn to -30 degrees
    while(angle < 2 * PI * 21 / 8){
        linear_time_step(x,y,angle,time, linear_speed,sampling_period,true_flight_path);
        angle += linear_speed / turning_radius * sampling_period;
    }
    while((x > 10000) && (y > 10000)){
        linear_time_step(x,y,angle,time, linear_speed,sampling_period,true_flight_path);
        linear_speed += 0.03 * sampling_period;
    }
    true_flight_path.close();
}

void inject_AWGN_and_quantize(double position_stdev, double velocity_stdev, double position_quantization, double velocity_quantization){
    
    ifstream true_flight_path;
    ofstream noisy_flight_path;
    true_flight_path.open ("true_flight_path.txt");
    noisy_flight_path.open ("noisy_flight_path.txt");
    vector<double> time;
    vector<double> x;
    vector<double> y;
    vector<double> speed;
    double time_t, x_t, y_t, speed_t;
    
    long long seed = std::chrono::system_clock::now().time_since_epoch().count();
    std::default_random_engine generator (seed);
    std::normal_distribution<double> pos_dist (0.0,position_stdev);
    std::normal_distribution<double> vel_dist (0.0,velocity_stdev);
    int i = 0;
    
    while(true_flight_path >> time_t >> x_t >> y_t >> speed_t)
    {
        time.push_back(time_t);
        x.push_back(x_t + pos_dist(generator));
        y.push_back(y_t + pos_dist(generator));
        speed.push_back(speed_t + vel_dist(generator));
        noisy_flight_path << setprecision(10) << time[i] << " " << setprecision(position_quantization) << x[i] << " " << y[i] << " ";
        noisy_flight_path << setprecision(velocity_quantization) << speed[i] << endl;
        ++i;
    }
    

}


int main(int argc, const char * argv[]) {
    ifstream sim_parameters;
    sim_parameters.open ("input.config");

    string dummystring;
    double position_stdev, velocity_stdev;
    int position_quantization, velocity_quantization;

    sim_parameters >> dummystring;
    sim_parameters >> position_stdev;
    sim_parameters >> dummystring;
    sim_parameters >> velocity_stdev;
    sim_parameters >> dummystring;
    sim_parameters >> position_quantization;
    sim_parameters >> dummystring;
    sim_parameters >> velocity_quantization;


    generate_flight_path();
    cout << position_stdev << " " <<velocity_stdev << " " << position_quantization << " " << velocity_quantization << endl;
    inject_AWGN_and_quantize(position_variance, velocity_variance, position_quantization, velocity_quantization);
}
