#ifndef MeasurementModel_h
#define MeasurementModel_h


#include "Particle.h"

//purely virtual base class for all models. Any child class needs to have three functions:
//1. Sample - samples the distribution given the parameters
//2. Compute transition probabilities - find the probability of passing from state x[n] to state x[n+1]
//3. Compute measurement probabilities - find the probability of measuring y given state x

class MeasurementModel{
public:
    virtual void sample(std::vector<double> & blank_vector) = 0;
    virtual void compute_transition_probability(const std::vector<Particle *> & input, std::vector<double> & output){}
    virtual void compute_measurement_probability(const std::vector<Particle *> & input, const Observation & ops, std::vector<double> & output){}
};

//Gaussian Observation Model
//Needs mean and stdev parameters for definition
class IndependentMVGaussianModel : public MeasurementModel{
    std::vector <double> mean;
    std::vector <double> stdev;
    std::vector <std::normal_distribution<double> >dist;
    int dimension;
    std::mt19937 gen;

public:
    IndependentMVGaussianModel(int dimension_);
    IndependentMVGaussianModel(int dimension_, std::vector<double> mean_, std::vector<double> stdev_);
    void sample(std::vector<double> & blank_vector);
    void compute_transition_probability(const std::vector<Particle *> & input, std::vector<double> & output);
    void compute_measurement_probability(const std::vector<Particle *> & input, const Observation & ops, std::vector<double> & output);
};

#endif /* MeasurementModel_h */
