#include "Particle.h"
using namespace std;

Particle::Particle(){}

Particle::Particle(vector<double> & state_, double weight_) :weight(weight_){
    state = state_;
}

Particle::Particle(int size){
    vector<double> state_(size,0);
    state = state_;
    weight = 1;
}
Particle::Particle(const Particle &p2){
    this->state = p2.state;
    this->weight = p2.weight;
}

//setters
void Particle::set_weight(double new_weight){ weight = new_weight; }

//getters
double Particle::get_weight(){ return weight; }

void Observation::set_state_value(double new_val, int index){ state[index] = new_val; }
double Observation::get_state_value(int index){ return state[index]; }

void Observation::set_state(vector<double> & state_){
    state = state_;
}
void Observation::get_state(vector<double> & state_){
    state_ = state;
}

void Observation::increment_state_value(double value, int index){
    state[index] += value;
}

vector<double> Observation::get_state_copy(){
    return state;
}

Observation::Observation(){
    vector<double> state_(dimension,0);
    state = state_;
}
Observation::Observation(int dimension_){
    state.resize(dimension_);
    dimension = dimension_;
    for (int i = 0; i < dimension_; ++i) {
        state[i] = 0;
    }
}

Observation::Observation(double * state_, int dimension_){
    state.resize(dimension_);
    dimension = dimension_;
    for (int i = 0; i < dimension_; ++i) {
        state[i] = state_[i];
    }
}


Observation::Observation(vector<double> & state_){
    state = state_;
}
Observation::Observation(const Observation &ops2){
    state= ops2.state;
}

void Observation::operator+=(const Observation & p){    // return type is a vector of integers
    if(p.state.size() != state.size()){
        std::cout << "Vectors of different sizes cannot add" << std::endl;
        exit(1);
    }
    for(int i=0; i < state.size(); i++){
        state[i] += p.state.at(i);
    }
}


void Observation::operator/=(const double & denominator){    // return type is a vector of integers
    
    for(int i=0; i < state.size(); i++){
        state[i] /= denominator;
    }
}

Observation operator + (const Observation& op1, const Observation& op2){
    if(op1.state.size() != op2.state.size()){
        std::cout << "Vectors of different sizes cannot add" << std::endl;
        exit(1);
    }
    Observation result;
    for(int i=0; i < op1.state.size(); i++){
        result.set_state_value(op1.state.at(i) + op2.state.at(i), i);
    }
    return result;
}

Observation operator - (const Observation& op1, const Observation& op2){
    if(op1.state.size() != op2.state.size()){
        std::cout << "Vectors of different sizes cannot add" << std::endl;
        exit(1);
    }
    Observation result;
    for(int i=0; i < op1.state.size(); i++){
        result.set_state_value(op1.state.at(i) - op2.state.at(i), i);
    }
    return result;
}
Observation squared_difference(const std::vector<double>& op1, const std::vector<double>& op2){
    if(op1.size() != op2.size()){
        std::cout << "Vectors of different sizes cannot add" << std::endl;
        exit(1);
    }
    Particle result;
    for(int i=0; i < op1.size(); i++){
        result.set_state_value(pow(op1.at(i) - op2.at(i), 2), i);
    }
    return result;
}

void load_true_states(vector<Observation> & true_states, string filename){
    ifstream true_flight_path;
    true_flight_path.open (filename);
    
    //create vector of observations
    double my_t;
    double my_params [4];
    string throwaway;
    getline(true_flight_path, throwaway); //first line is header
    while(true_flight_path >> my_t){
        true_flight_path >> my_params[0];
        true_flight_path >> my_params[1];
        true_flight_path >> my_params[2];
        true_flight_path >> my_params[3];
        true_states.emplace_back(my_params, 4);
    }
}
