#ifndef ParticleFilter_hpp
#define ParticleFilter_hpp

#include <stdio.h>
#include <vector>
#include <string>
#include <iostream>
#include <fstream>
#include <array>
#include "math.h"
#include "MeasurementModel.h"

class ParticleFilter{
    std::vector<Particle *> particles;
    std::vector<Observation> observations;
    
    MeasurementModel * measurement_model;
    MeasurementModel * transition_model;
    int dimension;
    int numParticles = 100;
    double dt = 0.2;
    long numObservations;
    std::vector<Observation> predicted_states;
    std::string location_prediction_mode = "Mean";
    std::string resampling_policy = "Fixed_Interval";
    int resampling_interval = 1;
    int resampling_interval_clock = 0;
    double num_effective_particle_ceiling = 0;
    std::mt19937 gen;

public:
    //Constructors
    ParticleFilter(int numParticles_ = 100, int dimension_ = 4);
    ParticleFilter(MeasurementModel * measurement_model, MeasurementModel * transition_model, int myNumParticles = 100);
    ~ParticleFilter();
    void set_measurement_model(MeasurementModel * measurement_model_);
    void set_transition_model(MeasurementModel * transition_model_);
    void set_resampling_policy(std::string resampling_policy_, int resampling_interval_, double num_effective_particle_ceiling_);
    //Assign weights to particles based off position and observation
    void weight_particles(long index_num);
    void weighting_function(Particle & particle);
    void read_input_data(std::string filename, double * return_val);
    void predict_state(long index_num);
    void initialize_particles();
    void time_step();
    long get_numObservations(){return numObservations;}
    std::vector<double> compute_filtered_error(std::vector<Observation> & true_states,std::string errortype);
    std::vector<double> compute_raw_error(std::vector<Observation> & true_states,std::string errortype);
    void resample_particles();
    Observation get_prediction(int i);
    void print_output(std::string filename);
    void compute_mean(std::vector<Particle> * particles, Particle & avg_particle);
    void compute_stdev(std::vector<Particle> * particles, Particle & sample_mean_particle, Particle & stdev_particle);
    void set_location_prediction_mode(std::string mode);
    double get_dt();
    int get_dimension();
};

#endif /* ParticleFilter_hpp */
